unit hiGL_Main;

interface

{$I share.inc}

uses Windows,Kol,Share,Debug,OpenGL;
function wglGetProcAddress(ProcName:PChar): Pointer; stdcall;
  var
  // VSync
  wglSwapIntervalEXT    : function (interval: GLint): Boolean; stdcall;
  wglGetSwapIntervalEXT : function: GLint; stdcall;
const
  GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT = $84FF;
   VSyncOn  = 1;
   VSyncOff = 0;

type
  THIGL_Main = class(TDebug)
   private
    hrc: HGLRC;
    DC:HDC;
    fps_time: Integer;
    fps_cur : Integer;
    g_FPS   : Integer;
    MaxAnisotropy:Integer;
    
   public
    _data_Handle:THI_Event;
    _prop_Color:TColor;
    _prop_TwoSide:boolean;
    _prop_ClearStencil:integer;
    _prop_StencilMask:integer;
    _prop_StencilBits:integer;

    _prop_Fovy:real;
    _prop_zNear:real;
    _prop_zFar:real;
    _prop_TexFilter:boolean;

    _event_onInit:THI_Event;
    _event_onViewPort:THI_Event;
    _event_onVSync:THI_Event;
    _event_onExtensions:THI_Event;

    destructor Destroy; override;
    procedure _work_doInit(var _Data:TData; Index:word);
    procedure _work_doViewPort(var _Data:TData; Index:word);
    procedure _work_doFlip(var _Data:TData; Index:word);
    procedure _work_doColor(var _Data:TData; Index:word);
    procedure _work_doVSync(var _Data:TData; Index:word);
    procedure _var_GLHandle(var _Data:TData; Index:word);
    procedure _var_Fps(var _Data:TData; Index:word);
    procedure _var_MaxAnisotropy(var _Data:TData; Index:word);
    procedure _var_Vendor(var _Data:TData; Index:word);
    procedure _var_Renderer(var _Data:TData; Index:word);
    procedure _var_VersionGL(var _Data:TData; Index:word);
  end;

   procedure glColor(C:TColor);
   procedure glColora(C:TColor; Alpha:real);

implementation

{$ifdef F_P}
function wglCreateContext(DC: HDC): HGLRC; stdcall; external opengl32 name 'wglCreateContext';
function wglMakeCurrent(DC: HDC; p2: HGLRC): BOOL; stdcall; external opengl32 name 'wglMakeCurrent';
function SwapBuffers(DC: HDC): BOOL; stdcall; external gdi32 name 'SwapBuffers';
function wglDeleteContext(p1: HGLRC): BOOL; stdcall; external opengl32 name 'wglDeleteContext';
{$endif}
function wglGetProcAddress; external 'opengl32';

procedure glColor(C:TColor);
begin
   with TRGB(c) do
     glColor3f(r/255, g/255, b/255);
end;

procedure glColora;
begin
   with TRGB(c) do
     glColor4f(r/255, g/255, b/255,alpha);
end;

destructor THIGL_Main.Destroy;
begin
    wglMakeCurrent(0, 0);
    wglDeleteContext(hrc);
    inherited;
end;

procedure THIGL_Main._work_doInit;
var
    nPixelFormat: Integer;
    pfd: TPixelFormatDescriptor;
    Extensions: string;
begin
   //InitOpenGL;
   //InitGLExtensions;

   DC := GetDC(ReadInteger(_Data,_data_handle,0));

   FillChar(pfd, SizeOf(pfd), 0);
  With pfd do
   begin
    dwFlags   := PFD_DRAW_TO_WINDOW or
                 PFD_SUPPORT_OPENGL or
                 PFD_DOUBLEBUFFER;
    cStencilBits := _prop_StencilBits;
    cDepthBits:= 32;
   end;
   nPixelFormat := ChoosePixelFormat(DC, @pfd);
   SetPixelFormat(DC, nPixelFormat, @pfd);

   hrc := wglCreateContext(DC);
   //hrc := CreateRenderingContext(DC,
   wglMakeCurrent(DC, hrc);

   //_debug(int2str(hrc));
   glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   //glShadeModel (GL_FLAT);

   with TRGB(_prop_Color) do
     glClearColor (r/255, g/255, b/255, 0.0);

   glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
   if _prop_TexFilter then
     begin
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
     end;
   //glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST );

   glClearStencil(_prop_ClearStencil);
   glStencilMask(_prop_StencilMask);

   if _prop_TwoSide then
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1);
    
    glGetIntegerv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, @MaxAnisotropy);
    _hi_CreateEvent(_Data,@_event_onInit);
     Extensions:= glGetString(GL_EXTENSIONS);
    _hi_onEvent(_event_onExtensions, Extensions);
end;

procedure THIGL_Main._work_doViewPort;
var h:cardinal;
    r:TRect;
begin
     h := ReadInteger(_Data,_data_handle,0);
     GetClientRect(h,r);
     wglMakeCurrent(DC, hrc);
     glViewport(0,0,r.Right - r.Left,r.Bottom - r.Top);
     _hi_OnEvent(_event_onViewPort);

     InvalidateRect(h, nil, False);
end;
//   
function GetTime: Integer;
var
  T : LARGE_INTEGER;
  F : LARGE_INTEGER;
begin
  QueryPerformanceFrequency(Int64(F));
  QueryPerformanceCounter(Int64(T));
  Result := Trunc(1000 * T.QuadPart / F.QuadPart);
end;

procedure THIGL_Main._work_doFlip;
begin
  wglMakeCurrent(DC, hrc);
  SwapBuffers(DC);
  //  -   
  if fps_time <= GetTime then
  begin
    fps_time := GetTime + 1000;
    g_FPS    := fps_cur;
    fps_cur  := 0;
  end;
  inc(fps_cur);
end;

procedure THIGL_Main._work_doColor;
var c:TColor;
begin
    c := ToInteger(_Data);
    with TRGB(c) do
     glClearColor (r/255, g/255, b/255, 0.0);
end;

procedure THIGL_Main._work_doVSync;
var
vs : Integer;
begin
    wglSwapIntervalEXT    := wglGetProcAddress('wglSwapIntervalEXT');
    wglGetSwapIntervalEXT := wglGetProcAddress('wglGetSwapIntervalEXT');
    vs := ToInteger(_Data);
    if vs = 0 then
    wglSwapIntervalEXT(0);
    if vs = 1 then
    wglSwapIntervalEXT(1);
    _hi_onEvent(_event_onVSync,wglGetSwapIntervalEXT);
end;

procedure THIGL_Main._var_GLHandle(var _Data:TData; Index:word);
begin
   dtInteger(_Data,integer(DC));
end;

procedure THIGL_Main._var_Fps(var _Data:TData; Index:word);
begin
   dtInteger(_Data,g_FPS);
end;
  
procedure THIGL_Main._var_MaxAnisotropy(var _Data:TData; Index:word);
begin
   dtInteger(_Data,MaxAnisotropy);
end;

procedure THIGL_Main._var_Vendor(var _Data:TData; Index:word);
var
   Vendor: string;
begin
   Vendor:= glGetString(GL_VENDOR);
   dtString(_Data,Vendor);
end;

procedure THIGL_Main._var_Renderer(var _Data:TData; Index:word);
var
   Renderer: string;
begin
   Renderer:= glGetString(GL_RENDERER);
   dtString(_Data,Renderer);
end;

procedure THIGL_Main._var_VersionGL(var _Data:TData; Index:word);
var
   VersionGL: string;
begin
   VersionGL:= glGetString(GL_VERSION);
   dtString(_Data,VersionGL);
end;
  
end.

